@@ -1,10 +1,12 @@
-from .schema import Schema, Attr, Link, Embedded
+from .schema import Schema, Attr, Link, Curie, Embedded, Accessor
 from . import types
 
 __all__ = [
+    "Accessor",
     "Schema",
     "Attr",
     "Link",
+    "Curie",
     "Embedded",
     "types",
 ]
